/*
* Copyright (c) 2005-08, Craig Bishop and Colin Graeme Johnson
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the names of Craig Bishop and/or Colin Graeme Johnson, 
*       nor the name of the University of Kent may be used to endorse 
*       or promote products derived from this software without
*       specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY COLIN GRAEME JOHNSON ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL COLIN GRAEME JOHNSON BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * Created on 06-Jul-2005
 */
package main.progAnal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Set;

/**
 * Class to check conditions under which variables are assigned and used
 * 
 * @author cbishop
 */
public class ConditionChecker extends ProgramAnalyser {

	private String variable;

	private LinkedHashMap analysedMap;

	private ArrayList methods;

	/**
	 * Constructor for ConditionChecker
	 * 
	 * @param var
	 *            Name of input variable
	 * @param inputMap
	 *            Analysed map of variable statements
	 * @param methodNames
	 *            List of method names in calss
	 */
	public ConditionChecker(LinkedHashMap inputMap, String var,
			ArrayList methodNames) {
		variable = var;
		analysedMap = inputMap;
		methods = methodNames;
	}

	/**
	 * Return whether there is no assignment statement for variable
	 * 
	 * @return boolean true is there is no assignment statement
	 */
	public boolean noAssignmentStatement() {
		boolean noAssignment = false;
		HashMap assignmentWhereaboutsMap = (HashMap) analysedMap
				.get("assignment");
		if (assignmentWhereaboutsMap.size() < 1) {
			noAssignment = true;
		}
		return noAssignment;
	}

	/**
	 * Return whether there is only one assignment statement for the variable in
	 * the program
	 * 
	 * @return boolean true if there is only one assignment statement
	 */
	public boolean onlyOneAssignmentStatement() {
		boolean onlyOneAssignment = false;
		HashMap assignmentWhereaboutsMap = (HashMap) analysedMap
				.get("assignment");
		if (assignmentWhereaboutsMap.size() <= 1) {
			onlyOneAssignment = true;
		}
		return onlyOneAssignment;
	}

	/**
	 * Return map of statements found in loops or branches
	 * 
	 * @param what
	 *            String being the category of statement
	 * @param where
	 *            String signifying whether to look in loops or branches
	 * @return HashMap containing statement and loop/branch statement in which
	 *         it is found
	 */
	public HashMap foundIn(String what, String where) {
		HashMap foundIn = new HashMap();
		HashMap whereaboutsMap = (HashMap) analysedMap.get(what);
		Set statementSet = whereaboutsMap.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			ArrayList statementIn = new ArrayList();
			String statement = (String) it.next();//get statement
			ArrayList specificOccuranceIn = new ArrayList();
			//get array for given statement
			ArrayList occuranceArray = (ArrayList) whereaboutsMap
					.get(statement);
			boolean found = false;
			if (loop(statement) && where.equals("loop")) {
				//add line number of start of loop to array
				specificOccuranceIn.add(occuranceArray.get(1));
				//add loop statement to array
				specificOccuranceIn.add(statement);
				found = true;
			}
			//get index for statement array
			ArrayList indexArray = (ArrayList) occuranceArray.get(0);
			for (int j = 0; j < indexArray.size() - 1; j++) {
				//get position in array of statement line number
				Integer index = (Integer) indexArray.get(j);
				//get posiition in array of next statement line number
				Integer nextIndex = (Integer) indexArray.get(j + 1);
				for (int i = index.intValue() + 1; i < nextIndex.intValue(); i += 2) {
					//get branch, loop, method, etc within which assignment
					// found
					String statementString = (String) occuranceArray.get(i + 1);
					if (where.equals("loop")) {
						if (loop(statementString)) { //isolate loops
							//add line number of start of loop to array
							specificOccuranceIn.add(occuranceArray.get(i));
							//add loop statement to array
							specificOccuranceIn.add(statementString);
							found = true;
						}
					} else if (where.equals("branch")) {
						if (branch(statementString)) {
							//add line number of start of loop to array
							specificOccuranceIn.add(occuranceArray.get(i));
							//add loop statement to array
							specificOccuranceIn.add(statementString);
							found = true;
						}
					}
				}
				if (found) {
					//if assignment in loop add line number of assignment
					// statement
					specificOccuranceIn.add(0, occuranceArray.get(index
							.intValue()));
					statementIn.add(specificOccuranceIn);
				}
			}
			if (found) {
				foundIn.put(statement, statementIn);
			}
		}
		return foundIn;
	}

	/**
	 * Return variable statements occuring in assignment loops together with
	 * loop signatures and line numbers
	 * 
	 * @param assignmentsInLoop
	 *            HashMap of assignment statements found in loops
	 * @param inputLoopStatements
	 *            HashMap of loop statements
	 * @return HashMap containing assignment statements and which loops they are
	 *         in
	 */
	public HashMap foundInAssignmentLoop(HashMap assignmentsInLoop,
			HashMap inputLoopStatements) {
		HashMap foundInAssignmentLoop = new HashMap();
		Set statementSet = inputLoopStatements.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next();
			ArrayList statementInLoop = (ArrayList) inputLoopStatements
					.get(statement);
			ArrayList specificStatement = compareLoopStatements(
					assignmentsInLoop, statementInLoop);
			if (specificStatement.size() > 0) {
				foundInAssignmentLoop.put(statement, specificStatement);
			}
		}
		return foundInAssignmentLoop;
	}

	/*
	 * Return array list containing occurance arrays for a given statement
	 */
	private ArrayList compareLoopStatements(HashMap assignmentsInLoop,
			ArrayList statementInLoop) {
		ArrayList specificStatement = new ArrayList();
		Set statementSet = assignmentsInLoop.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next();
			ArrayList assignStatInLoop = (ArrayList) assignmentsInLoop
					.get(statement);
			for (int i = 0; i < statementInLoop.size(); i++) {
				for (int j = 0; j < assignStatInLoop.size(); j++) {
					ArrayList compares = (ArrayList) statementInLoop.get(i);
					ArrayList assigns = (ArrayList) assignStatInLoop.get(j);
					ArrayList inAssignLoop = lookForCommon(compares, assigns);
					if (inAssignLoop.size() > 0) {
						inAssignLoop.add(2, statement); //add assignment
						// statement to array
						specificStatement.add(inAssignLoop);
					}
				}
			}
		}
		return specificStatement;
	}

	/*
	 * Return array list containing details of occurance for given statement
	 */
	private ArrayList lookForCommon(ArrayList compares, ArrayList assigns) {
		ArrayList returnArray = new ArrayList();
		boolean found = false;
		for (int i = 1; i < compares.size(); i += 2) {
			Integer comp = (Integer) compares.get(i);
			for (int j = 1; j < assigns.size(); j += 2) {
				Integer ass = (Integer) assigns.get(j);
				if (comp.intValue() == ass.intValue()) {
					found = true;
					returnArray.add(comp);
					returnArray.add(compares.get(i + 1));
				}
			}
		}
		if (found) {
			//add line number of compare statement
			returnArray.add(0, compares.get(0));
			//add line number of assign statement
			returnArray.add(1, assigns.get(0));

		}
		return returnArray;
	}

	/**
	 * Return ArrayList of statements where use in assignment loop is directly
	 * or indirectly related to loop condition
	 * 
	 * @param foundInAssignmentLoop
	 *            HashMap of statement found in assignment loop
	 * @return ArrayList of statements that relate to loop condition
	 */
	public ArrayList useForLoopCondition(HashMap foundInAssignmentLoop) {
		ArrayList useForLoopCondition = new ArrayList();
		Set statementSet = foundInAssignmentLoop.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next();
			ArrayList statementInLoop = (ArrayList) foundInAssignmentLoop
					.get(statement);
			for (int i = 0; i < statementInLoop.size(); i++) {
				ArrayList specificOccurance = (ArrayList) statementInLoop
						.get(i);
				for (int j = 4; j < specificOccurance.size(); j += 2) {
					String loopStatement = (String) specificOccurance.get(j);
					if (statement.equals(loopStatement)) {
						useForLoopCondition.add(statement);
					}
				}
			}
		}
		return useForLoopCondition;
	}

	/**
	 * Return true if assignment loop is dependent on assignment statement
	 * following conditional use for a given variable
	 * 
	 * @param foundInAssignmentLoop
	 *            HashMap of statements found in assignment loop
	 * @return boolean
	 */
	public boolean indirectUse(HashMap foundInAssignmentLoop) {
		boolean indirectUse = false;
		Set statementSet = foundInAssignmentLoop.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next();
			ArrayList statementInLoop = (ArrayList) foundInAssignmentLoop
					.get(statement);
			for (int i = 0; i < statementInLoop.size(); i++) {
				ArrayList specificOccurance = (ArrayList) statementInLoop
						.get(i);
				for (int j = 4; j < specificOccurance.size(); j += 2) {
					String loopStatement = (String) specificOccurance.get(j);
					int bracketCount = 0;
					boolean afterBrackets = false;
					String indirectAssign = "";
					for (int k = 0; k < statement.length(); k++) {
						String statementSubstring = statement.substring(k);
						if (bracketCount == 0 && afterBrackets) {
							indirectAssign += statement.substring(k, k + 1);
							if (indirectAssign.endsWith("=")) {
								//remove "="
								indirectAssign = indirectAssign.substring(0,
										indirectAssign.length() - 1);
								indirectAssign = indirectAssign.trim();
								if (contains(statement, indirectAssign)
										&& !subString(statement, indirectAssign)
										&& statementSet.size() < 2) {
									indirectUse = true;
									break;
								}
							}
						}
						if (statementSubstring.startsWith("(")) {
							bracketCount++;
							afterBrackets = true;
						}
						if (statementSubstring.startsWith(")")) {
							bracketCount--;
						}
					}
				}
			}
		}
		return indirectUse;
	}

	/**
	 * Return list of statements where variable is assigned in a "for" loop
	 * declaration
	 * 
	 * @param what
	 *            String signfying the category of statement
	 * @return ArrayList of "for" loop assignments
	 */
	public ArrayList assignmentInFor(String what) {
		ArrayList assignInFor = new ArrayList();
		HashMap whereaboutsMap = (HashMap) analysedMap.get(what);
		Set statementSet = whereaboutsMap.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next();
			String noSpace = removeSpaces(statement);
			if (statement.startsWith("for") && inBrackets(noSpace, variable)
					&& contains(noSpace, variable + "=")
					&& !subString(statement, variable)) {
				assignInFor.add(statement);
			}
		}
		return assignInFor;
	}

	/**
	 * Return if variable is not used directly in the program
	 * 
	 * @return boolean
	 */
	public boolean noDirectUsage() {
		boolean noDirectUsage = false;
		HashMap usageWhereaboutsMap = (HashMap) analysedMap.get("usage");
		if (usageWhereaboutsMap.size() < 1) {
			noDirectUsage = true;
		}
		return noDirectUsage;
	}

	/**
	 * Return map containing assignment statements and branch statements where
	 * variable is condition for branch
	 * 
	 * @return HashMap
	 */
	public HashMap conditionAssignStatement() {
		HashMap conditionForAssignmentBranch = new HashMap();
		HashMap assignments = (HashMap) analysedMap.get("assignment");
		ArrayList conditionAssignment = new ArrayList();
		Set statementSet = assignments.keySet();
		Iterator it = statementSet.iterator();
		//check all assignment statements to see if they are branch statements
		while (it.hasNext()) {
			String assignmentStatement = (String) it.next();
			boolean conditionalAssignment = false;
			//check if variable is condition for its assignment
			if (branch(assignmentStatement)
					&& inBrackets(assignmentStatement, variable)) {
				ArrayList statementIn = new ArrayList();
				//get array for given statement
				ArrayList occuranceArray = (ArrayList) assignments
						.get(assignmentStatement);
				//add line number of assignment statement
				conditionAssignment.add(occuranceArray.get(1));
				//add line number of conditional statement
				conditionAssignment.add(occuranceArray.get(1));
				//add assignment statement to array list
				conditionAssignment.add(0, assignmentStatement);
				conditionalAssignment = true;
			}
			if (conditionalAssignment)
				conditionForAssignmentBranch.put(assignmentStatement,
						conditionAssignment);
		}
		return conditionForAssignmentBranch;
	}

	/**
	 * Return map containing incidences where assignment in branch is also in
	 * loop
	 * 
	 * @param assignmentsInBranch
	 *            HashMap containing assignmentsInBranch
	 * @param inputLoopStatements
	 *            HashMap containing loop statements
	 * @return HashMap
	 */
	public HashMap conditionForAssignmentBranch(HashMap assignmentsInBranch,
			HashMap inputLoopStatements) {
		HashMap conditionForAssignmentBranch = new HashMap();
		Set inputSet = inputLoopStatements.keySet();
		Iterator it = inputSet.iterator();
		while (it.hasNext()) {
			String statement = (String) it.next(); //get statement to look for
			Set branchAssignments = assignmentsInBranch.keySet();
			Iterator iter = branchAssignments.iterator();
			ArrayList conditionAssignment = new ArrayList();
			while (iter.hasNext()) {
				String assignStatement = (String) iter.next();
				ArrayList statementInBranch = (ArrayList) assignmentsInBranch
						.get(assignStatement);
				for (int i = 0; i < statementInBranch.size(); i++) {
					ArrayList specificOccurance = (ArrayList) statementInBranch
							.get(i);
					//add assignment statement to array list
					conditionAssignment.add(assignStatement);
					//add line number for assignment statement
					conditionAssignment.add(specificOccurance.get(0));
					boolean isCommon = false;
					for (int j = 2; j < specificOccurance.size(); j += 2) {
						String conditionalStatement = (String) specificOccurance
								.get(j);
						if (statement.equals(conditionalStatement)) {
							//add line number for branch
							conditionAssignment.add(specificOccurance
									.get(j - 1));
							isCommon = true;
							j = specificOccurance.size();
						}
					}
					//if no common statement found, remove assignment statement
					// and line number
					if (!isCommon) {
						conditionAssignment
								.remove(conditionAssignment.size() - 1);
						conditionAssignment
								.remove(conditionAssignment.size() - 1);
					}
				}
			}
			//check if assignment in branches for which variable is condition
			if (conditionAssignment.size() > 0) {
				conditionForAssignmentBranch
						.put(statement, conditionAssignment);
			}
		}
		return conditionForAssignmentBranch;
	}

	/**
	 * Return list of statement of a given type
	 * 
	 * @param assignmentsInLoop
	 *            HashMap of assignment statements found in loops
	 * @param statementType
	 *            String being the type of statement to look for
	 * @return ArrayList of statements
	 */
	public ArrayList statementTypeCheck(HashMap assignmentsInLoop,
			String statementType) {
		ArrayList returnArray = new ArrayList();
		Set statementSet = assignmentsInLoop.keySet();
		Iterator it = statementSet.iterator();
		//check all assignment statements to see if variable appears on both
		// sides of statement
		while (it.hasNext()) {
			String statement = (String) it.next();
			if (statementType.equals("bothSides") && onBothSides(statement)) {
				returnArray.add(statement);
			} else if (statementType.equals("indirect")
					&& indirectBothSides(statement)) {
				returnArray.add(statement);
			} else if (statementType.equals("toggle")
					&& toggleStatement(statement)) {
				returnArray.add(statement);
			} else if (statementType.equals("incDec") && incDecEtc(statement)) {
				returnArray.add(statement);
			}
		}
		return returnArray;
	}

	/*
	 * Check if variable appears on both sides of assignment statement
	 */
	private boolean onBothSides(String statement) {
		boolean onBothSides = false;
		String afterEquals = afterEquals(statement);
		if (contains(afterEquals, variable) && !contains(afterEquals, ",")
				&& !subString(afterEquals, variable)) {
			onBothSides = true;
		}
		return onBothSides;
	}

	/*
	 * Check if variable appears in directly on both sides of assignment
	 * statement
	 */
	private boolean indirectBothSides(String statement) {
		boolean indirectBothSides = false;
		int bracketCount = 0;
		for (int k = 0; k < statement.length(); k++) {
			String statementSubstring = statement.substring(k);
			if (statementSubstring.startsWith(variable) && bracketCount == 0) {
				String afterVar = statementSubstring.substring(variable
						.length());
				afterVar = removeSpaces(afterVar);
				if (afterVar.startsWith("+=") || afterVar.startsWith("-=")
						|| afterVar.startsWith("*=")
						|| afterVar.startsWith("/=")
						|| afterVar.startsWith("%=")
						&& !arithExp(afterVar.substring(2))) {
					indirectBothSides = true;
					k = statement.length(); //end loop if incDec detected
				}
			}
			if (statementSubstring.startsWith("(")) {
				bracketCount++;
			}
			if (statementSubstring.startsWith(")")) {
				bracketCount--;
			}
		}
		return indirectBothSides;
	}

	/*
	 * Return true if variable is toggled within statement
	 */
	private boolean toggleStatement(String statement) {
		boolean toggleStatement = false;
		String afterEquals = afterEquals(statement);
		if (contains(afterEquals, "!" + variable)
				&& !subString(afterEquals, "!" + variable)) {
			toggleStatement = true;
		}
		return toggleStatement;
	}

	/*
	 * Return true if variable is incremented/decremented in statement
	 */
	private boolean incDecEtc(String statement) {
		boolean incDecStatement = false;
		int bracketCount = 0;
		for (int k = 0; k < statement.length(); k++) {
			String statementSubstring = statement.substring(k);
			if (statementSubstring.startsWith(variable) && bracketCount == 0) {
				String afterVar = statementSubstring.substring(variable
						.length());
				afterVar = removeSpaces(afterVar);
				if (afterVar.startsWith("++")
						|| afterVar.startsWith("--")
						|| afterVar.startsWith("**")
						|| afterVar.startsWith("//")
						|| (afterVar.startsWith("+=")
								|| afterVar.startsWith("-=")
								|| afterVar.startsWith("*=")
								|| afterVar.startsWith("/=") || afterVar
								.startsWith("%=")
								&& arithExp(afterVar.substring(2)))
						|| (afterVar.startsWith("=") && onBothSides(statement) && arithExp(
								afterVar.substring(1), variable))) {
					incDecStatement = true;
					k = statement.length(); //end loop if incDec detected
				}
			}
			if (statementSubstring.startsWith("(")) {
				bracketCount++;
			}
			if (statementSubstring.startsWith(")")) {
				bracketCount--;
			}
		}
		return incDecStatement;
	}

	/**
	 * Return map of statements where variable is used outside of the loop in
	 * which it is assigned
	 * 
	 * @param inAssignLoop
	 *            HashMap containing usage/conditional statement found in
	 *            assignment loop
	 * @param what
	 *            String being category of statement
	 * @return HashMap
	 */
	public HashMap outsideAssignLoop(HashMap inAssignLoop, String what) {
		HashMap outsideAssignLoop = new HashMap();
		HashMap whereaboutsMap = (HashMap) analysedMap.get(what);
		Set statementSet = whereaboutsMap.keySet();
		Iterator it = statementSet.iterator();
		while (it.hasNext()) {
			ArrayList statementIn = new ArrayList();
			String statement = (String) it.next();//get statement
			boolean foundOutsideAssign = false;
			ArrayList specificOccuranceOutside = new ArrayList();
			if ((loop(statement) && what.equals("conditional"))
					|| what.equals("usage")) {
				//get array for given statement
				ArrayList occuranceArray = (ArrayList) whereaboutsMap
						.get(statement);
				//get index for statement array
				ArrayList indexArray = (ArrayList) occuranceArray.get(0);
				for (int j = 0; j < indexArray.size() - 1; j++) {
					//get position in array of statement line number
					Integer index = (Integer) indexArray.get(j);
					//get branch, loop, method, etc within which assignment
					// found
					Integer statementLineNo = (Integer) occuranceArray
							.get(index.intValue());
					specificOccuranceOutside = foundOutside(statementLineNo,
							inAssignLoop);
				}
			}
			//if statement found outside assign loop
			if (specificOccuranceOutside.size() > 0) {
				//add array of line numbers for that statement to return map
				outsideAssignLoop.put(statement, specificOccuranceOutside);
			}
		}
		return outsideAssignLoop;
	}

	/*
	 * Return list of statements found outside assignment loop
	 */
	private ArrayList foundOutside(Integer statementLineNo,
			HashMap foundInAssignLoop) {
		ArrayList returnArray = new ArrayList();
		Set foundSet = foundInAssignLoop.keySet();
		Iterator it = foundSet.iterator();
		boolean matchFound = false;
		while (it.hasNext()) {
			//get statement found in assign loop
			String foundStatement = (String) it.next();
			//get array for given statement
			ArrayList statementInLoop = (ArrayList) foundInAssignLoop
					.get(foundStatement);
			for (int i = 0; i < statementInLoop.size(); i++) {
				//get array list for occurance of statement in loop
				ArrayList specificOccurance = (ArrayList) statementInLoop
						.get(i);
				//get line number of statement
				Integer statementInLoopLineNo = (Integer) specificOccurance
						.get(0);
				//see if input statement found in assign loop
				if (statementLineNo.intValue() == statementInLoopLineNo
						.intValue()) {
					matchFound = true;
				}
			}
		}
		if (!matchFound) { //if input statement line number not found in assign
			//loop add input statement line number to return array
			returnArray.add(statementLineNo);
		}
		return returnArray;
	}

	/**
	 * Return list of statements where variable appears to be being used like an
	 * array
	 * 
	 * @return ArrayList
	 */
	public ArrayList arrayCheck() {
		ArrayList isArray = new ArrayList();
		String[] whatArray = { "assignment", "usage", "conditional", "other" };
		for (int i = 0; i < whatArray.length; i++) {
			HashMap whereaboutsMap = (HashMap) analysedMap.get(whatArray[i]);
			Set statementSet = whereaboutsMap.keySet();
			Iterator it = statementSet.iterator();
			while (it.hasNext()) {
				String statement = (String) it.next();//get statement
				String noSpace = removeSpaces(statement);
				if (contains(noSpace, variable + "[")
						&& !subString(noSpace, variable)) {
					isArray.add(statement);
					break;
				} else if (contains(noSpace, "]" + variable)) {
					isArray.add(statement);
					break;
				}
			}
		}
		return isArray;
	}

	/**
	 * Return list of statements where variable appears to be being used as
	 * organizer
	 * 
	 * @return ArrayList
	 */
	public ArrayList reorganize() {
		ArrayList returnArray = new ArrayList();
		HashMap whereaboutsMap = (HashMap) analysedMap.get("assignment");
		Set statementSet = whereaboutsMap.keySet();
		Iterator it = statementSet.iterator();
		boolean notReorganize = false;
		while (it.hasNext()) {
			String statement = (String) it.next();//get statement
			String afterEquals = afterEquals(statement);
			String noSpace = removeSpaces(afterEquals);
			if (contains(noSpace, variable + "[")
					&& !subString(afterEquals, variable)
					&& !arithExp(noSpace, variable)) {
				returnArray.add(statement);
			} else if (contains(noSpace, variable)
					|| arithExp(noSpace, variable)
					&& !subString(afterEquals, variable)) {
				notReorganize = true;
				returnArray.clear();
				break;
			}
		}
		return returnArray;
	}

	/**
	 * Return list of statements where variable appears to be being used as
	 * transformation
	 * 
	 * @param assignmentsInLoop
	 * 				HashMap containing assignments in loops
	 * @return ArrayList
	 */
	public ArrayList transform(HashMap assignmentsInLoop) {
		ArrayList returnArray = new ArrayList();
		Set statementSet = assignmentsInLoop.keySet();
		Iterator it = statementSet.iterator();
		boolean isTransform = true;
		while (it.hasNext()) {
			String statement = (String) it.next();
			returnArray.add(statement);
			String afterEquals = afterEquals(statement);
			for (int i = 0; i < methods.size(); i++) {
				String method = (String) methods.get(i);
				//check if method is called on right hand side of assignment
				if (contains(afterEquals, method + "(")) {
					//if any statement is not transformation clear array and
					// end method
					returnArray.clear();
					break;
				}
			}
			//if any statement is not transformation clear array and end method
			if (!is(afterEquals, "operator")) {
				returnArray.clear();
				break;
			}
		}
		return returnArray;
	}

	/**
	 * Return list of statements in which variable is indirectly toggled by
	 * being set twice within a loop, once in a nested loop, with opposing
	 * values
	 * 
	 * @param assignmentsInLoop
	 *            HashMap of assignment statements in loop
	 * @return ArrayList
	 */
	public ArrayList nestedBooleanAssign(HashMap assignmentsInLoop) {
		ArrayList returnArray = new ArrayList();
		Set statementSet = assignmentsInLoop.keySet();
		Iterator it = statementSet.iterator();
		boolean isBoolean = false;
		boolean isTrue = false;
		ArrayList compareArray = new ArrayList();
		while (it.hasNext()) {
			String statement = (String) it.next();
			String afterEquals = afterEquals(statement);
			if (contains(afterEquals, "true")
					&& !subString(afterEquals, "true")) {
				isBoolean = true;
				isTrue = true;
			} else if (contains(afterEquals, "false")
					&& !subString(afterEquals, "false")) {
				isBoolean = true;
			}
			if (isBoolean) {
				ArrayList statementInLoop = (ArrayList) assignmentsInLoop
						.get(statement);
				boolean isNested = false;
				for (int i = 0; i < statementInLoop.size(); i++) {
					ArrayList specificOccurance = (ArrayList) statementInLoop
							.get(i);
					if (specificOccurance.size() > 3) {
						isNested = true;
					}
					compareArray.add(statement); //add statement in question
					//add outer most line number of loop
					compareArray.add(specificOccurance.get(1));
					//add whether assignment is true or false
					compareArray.add(new Boolean(isTrue));
					//add whether assign statement is in nest loop
					compareArray.add(new Boolean(isNested));
				}
			}
		}
		if (compareArray.size() > 4) {
			returnArray = compareStatements(compareArray);
		}
		return returnArray;
	}

	/*
	 * Return list of statement stating whether the each assignment statement is
	 * in a nested loop or not
	 */
	private ArrayList compareStatements(ArrayList compareArray) {
		ArrayList returnArray = new ArrayList();
		for (int i = 0; i < compareArray.size(); i += 4) {
			Boolean isNested = (Boolean) compareArray.get(i + 3);
			if (isNested.booleanValue()) {
				Integer comparedLineNum = (Integer) compareArray.get(i + 1);
				Boolean comparedValue = (Boolean) compareArray.get(i + 2);
				//compare line number with loop lines for rested of array
				for (int j = i + 4; j < compareArray.size(); j += 4) {
					Boolean compareNested = (Boolean) compareArray.get(j + 3);
					Integer compareInteger = (Integer) compareArray.get(j + 1);
					Boolean compareValue = (Boolean) compareArray.get(j + 2);
					if (!compareNested.booleanValue()
							&& comparedLineNum.intValue() == compareInteger
									.intValue()
							&& comparedValue != compareValue) {
						returnArray.add(compareArray.get(i));
						returnArray.add("nested");
						returnArray.add(compareArray.get(j));
						break;
					}
				}
			} else {
				Integer comparedLineNum = (Integer) compareArray.get(i + 1);
				Boolean comparedValue = (Boolean) compareArray.get(i + 2);
				//compare line number with loop lines for rested of array
				for (int j = i + 4; j < compareArray.size(); j++) {
					Boolean compareNested = (Boolean) compareArray.get(j + 3);
					Integer compareInteger = (Integer) compareArray.get(j + 1);
					Boolean compareValue = (Boolean) compareArray.get(j + 2);
					if (compareNested.booleanValue()
							&& comparedLineNum.intValue() == compareInteger
									.intValue()
							&& comparedValue != compareValue) {
						returnArray.add(compareArray.get(i));
						returnArray.add("not nested");
						returnArray.add(compareArray.get(j));
						break;
					}
				}
			}
		}
		return returnArray;
	}

	/**
	 * Return list of statement in which variable is assigned with output from
	 * method call
	 * 
	 * @param assignmentsInLoop
	 *            HashMap of assignment statements found in loops
	 * @return ArrayList
	 */
	public ArrayList assignedWithMethod(HashMap assignmentsInLoop) {
		ArrayList returnArray = new ArrayList();
		Set statementSet = assignmentsInLoop.keySet();
		Iterator it = statementSet.iterator();
		boolean isTransform = true;
		while (it.hasNext()) {
			String statement = (String) it.next();
			String afterEquals = afterEquals(statement);
			for (int i = 0; i < methods.size(); i++) {
				String method = (String) methods.get(i);
				//check if method is called on right hand side of assignment
				if (contains(afterEquals, method + "(")) {
					returnArray.add(statement);
				}
			}
		}
		return returnArray;
	}

	/**
	 * Return list of statements where variable is assigned with value in loop
	 * before it is used in that loop
	 * 
	 * @param usageInAssignments
	 * @return ArrayList
	 */
	public ArrayList assignBeforeUse(HashMap usageInAssignments) {
		ArrayList returnArray = new ArrayList();
		Set usageSet = usageInAssignments.keySet();
		Iterator it = usageSet.iterator();
		while (it.hasNext()) {
			String usageStatement = (String) it.next();
			ArrayList usageInLoop = (ArrayList) usageInAssignments
					.get(usageStatement);
			for (int i = 0; i < usageInLoop.size(); i++) {
				ArrayList usageOccurance = (ArrayList) usageInLoop.get(i);
				Integer assLineNum = (Integer) usageOccurance.get(1);
				int assL = assLineNum.intValue();
				//get outer most loop line number
				Integer useLineNum = (Integer) usageOccurance.get(0);
				int useL = useLineNum.intValue();
				if (assLineNum.intValue() < useLineNum.intValue()) {
					returnArray.add(usageStatement);
					returnArray.add(usageOccurance.get(2));
					for (int j = 4; j < usageOccurance.size(); j += 2) {
						returnArray.add(usageOccurance.get(j));
					}
				} else {
					returnArray.clear();
				}
			}
		}
		return returnArray;
	}

	/**
	 * Return list of statements where variable is assigned with value from
	 * instantiation of new object, or assigned directly with boolean value
	 * 
	 * @param assignmentsInLoop
	 *            HashMap of assignment statement found in loops
	 * @return ArrayList
	 */
	public ArrayList otherAssignment(HashMap assignmentsInLoop) {
		ArrayList returnArray = new ArrayList();
		Set usageSet = assignmentsInLoop.keySet();
		Iterator it = usageSet.iterator();
		while (it.hasNext()) {
			String assignStatement = (String) it.next();
			String afterEquals = afterEquals(assignStatement);
			String noSpace = removeSpaces(afterEquals);
			if (is(afterEquals, "other")) {
				returnArray.add(assignStatement);
			}
			//check if value assigned by instantiating new object
			else if (contains(afterEquals, "new")
					&& !subString(afterEquals, "new")) {
				returnArray.add(assignStatement);
			} else if (noSpace.equals("=true;")) {
				returnArray.add(assignStatement);
			} else if (noSpace.equals("=false;")) {

			}
		}
		return returnArray;
	}
}